@@ -17,6 +17,8 @@ except ImportError:
 
 import calendar
 import pytz
+from binascii import b2a_hex
+
 from six import integer_types, iterkeys, text_type, PY3
 from six.moves import xrange
 
@@ -26,6 +28,10 @@ class MissingClassDefinition(ValueError):
         super(MissingClassDefinition, self).__init__("No class definition for class %s" % (class_name,))
 
 
+class UnknownSerializerError(ValueError):
+    pass
+
+
 class MissingTimezoneWarning(RuntimeWarning):
     def __init__(self, *args):
         args = list(args)
@@ -75,15 +81,15 @@ def import_classes_from_modules(*args):
                 import_class(item)
 
 
-def encode_object(obj, traversal_stack, generator_func):
+def encode_object(obj, traversal_stack, generator_func, on_unknown=None):
     values = obj.bson_encode()
     class_name = obj.__class__.__name__
     values["$$__CLASS_NAME__$$"] = class_name
-    return encode_document(values, traversal_stack, obj, generator_func)
+    return encode_document(values, traversal_stack, obj, generator_func, on_unknown)
 
 
-def encode_object_element(name, value, traversal_stack, generator_func):
-    return b"\x03" + encode_cstring(name) + encode_object(value, traversal_stack, generator_func=generator_func)
+def encode_object_element(name, value, traversal_stack, generator_func, on_unknown):
+    return b"\x03" + encode_cstring(name) + encode_object(value, traversal_stack, generator_func=generator_func, on_unknown=on_unknown)
 
 
 class _EmptyClass(object):
@@ -119,6 +125,10 @@ def decode_string(data, base):
 
 
 def encode_cstring(value):
+    if "\x00" in value:
+        raise ValueError("Element names may not include NUL bytes.")
+        # A NUL byte is used to delimit our string, accepting one would cause
+        # our string to terminate early.
     if isinstance(value, integer_types):
         value = str(value)
     if isinstance(value, text_type):
@@ -163,6 +173,7 @@ ELEMENT_TYPES = {
     0x03: "document",
     0x04: "array",
     0x05: "binary",
+    0x07: "object_id",
     0x08: "boolean",
     0x09: "UTCdatetime",
     0x0A: "none",
@@ -191,17 +202,17 @@ def decode_string_element(data, base):
     return base, name, value
 
 
-def encode_value(name, value, buf, traversal_stack, generator_func):
+def encode_value(name, value, buf, traversal_stack, generator_func, on_unknown=None):
     if isinstance(value, BSONCoding):
-        buf.write(encode_object_element(name, value, traversal_stack, generator_func))
+        buf.write(encode_object_element(name, value, traversal_stack, generator_func, on_unknown))
     elif isinstance(value, float):
         buf.write(encode_double_element(name, value))
     elif isinstance(value, text_type):
         buf.write(encode_string_element(name, value))
     elif isinstance(value, dict):
-        buf.write(encode_document_element(name, value, traversal_stack, generator_func))
+        buf.write(encode_document_element(name, value, traversal_stack, generator_func, on_unknown))
     elif isinstance(value, list) or isinstance(value, tuple):
-        buf.write(encode_array_element(name, value, traversal_stack, generator_func))
+        buf.write(encode_array_element(name, value, traversal_stack, generator_func, on_unknown))
     elif isinstance(value, str) or isinstance(value, bytes):
         buf.write(encode_binary_element(name, value))
     elif isinstance(value, bool):
@@ -218,9 +229,14 @@ def encode_value(name, value, buf, traversal_stack, generator_func):
                 buf.write(encode_int64_element(name, value))
             else:
                 buf.write(encode_int32_element(name, value))
+    else:
+        if on_unknown is not None:
+            encode_value(name, on_unknown(value), buf, traversal_stack, generator_func, on_unknown)
+        else:
+            raise UnknownSerializerError()
 
 
-def encode_document(obj, traversal_stack, traversal_parent=None, generator_func=None):
+def encode_document(obj, traversal_stack, traversal_parent=None, generator_func=None, on_unknown=None):
     buf = StringIO()
     key_iter = iterkeys(obj)
     if generator_func is not None:
@@ -228,19 +244,19 @@ def encode_document(obj, traversal_stack, traversal_parent=None, generator_func=
     for name in key_iter:
         value = obj[name]
         traversal_stack.append(TraversalStep(traversal_parent or obj, name))
-        encode_value(name, value, buf, traversal_stack, generator_func)
+        encode_value(name, value, buf, traversal_stack, generator_func, on_unknown)
         traversal_stack.pop()
     e_list = buf.getvalue()
     e_list_length = len(e_list)
     return struct.pack("<i%dsb" % (e_list_length,), e_list_length + 4 + 1, e_list, 0)
 
 
-def encode_array(array, traversal_stack, traversal_parent = None, generator_func = None):
+def encode_array(array, traversal_stack, traversal_parent = None, generator_func = None, on_unknown = None):
     buf = StringIO()
     for i in xrange(0, len(array)):
         value = array[i]
         traversal_stack.append(TraversalStep(traversal_parent or array, i))
-        encode_value(text_type(i), value, buf, traversal_stack, generator_func)
+        encode_value(text_type(i), value, buf, traversal_stack, generator_func, on_unknown)
         traversal_stack.pop()
     e_list = buf.getvalue()
     e_list_length = len(e_list)
@@ -258,7 +274,6 @@ def decode_document(data, base):
     length = struct.unpack("<i", data[base:base + 4])[0]
     end_point = base + length
     if data[end_point - 1] not in ('\0', 0):
-        print(type(data[end_point - 1]))
         raise ValueError('missing null-terminator in document')
     base += 4
     retval = {}
@@ -270,8 +285,8 @@ def decode_document(data, base):
     return end_point, retval
 
 
-def encode_document_element(name, value, traversal_stack, generator_func):
-    return b"\x03" + encode_cstring(name) + encode_document(value, traversal_stack, generator_func=generator_func)
+def encode_document_element(name, value, traversal_stack, generator_func, on_unknown):
+    return b"\x03" + encode_cstring(name) + encode_document(value, traversal_stack, generator_func=generator_func, on_unknown=on_unknown)
 
 
 def decode_document_element(data, base):
@@ -280,8 +295,8 @@ def decode_document_element(data, base):
     return base, name, value
 
 
-def encode_array_element(name, value, traversal_stack, generator_func):
-    return b"\x04" + encode_cstring(name) + encode_array(value, traversal_stack, generator_func=generator_func)
+def encode_array_element(name, value, traversal_stack, generator_func, on_unknown):
+    return b"\x04" + encode_cstring(name) + encode_array(value, traversal_stack, generator_func=generator_func, on_unknown=on_unknown)
 
 
 def decode_array_element(data, base):
@@ -359,3 +374,13 @@ def decode_int64_element(data, base):
     base, name = decode_cstring(data, base + 1)
     value = struct.unpack("<q", data[base:base + 8])[0]
     return base + 8, name, value
+
+
+def encode_object_id_element(name, value):
+    return b"\x07" + encode_cstring(name) + value
+
+
+def decode_object_id_element(data, base):
+    base, name = decode_cstring(data, base + 1)
+    value = b2a_hex(data[base:base + 12])
+    return base + 12, name, value
